/* Simple gdbm database dump -
     Copyright (c) 2002 - Burton M. Strauss III (BStrauss@acm.org)
     Released under GPL v2
 */

#include <stdio.h>
#include <string.h>
#include <gdbm.h>
int main(int argc, char *argv[]) {
   GDBM_FILE dbfile;
   datum key, data;
   int recordCount = 0, i, j, foundBinary, wideMode=0, npKey;
   unsigned char outC;
   char buf[43];

   printf("dumpgdbm - Burton Strauss - Burton@ntopSupport.com - v1.1\n");

   if ( (argc < 2) || (argc > 3) ) {
      fprintf (stderr, "Usage: dumpgdbm file [key]\n\n");
      exit (1);
   }
   dbfile = gdbm_open (argv[1], 0, GDBM_READER, 0666, NULL);
   if (!dbfile) {
      fprintf (stderr, "Open file %s, error %d (%s) gdbm file.\n", argv[1],
                       gdbm_errno, gdbm_strerror(gdbm_errno));
      exit (2);
   }

   if (argv[2] == NULL) {

       printf ("Dump of file %s...\n", argv[1]);
       printf ("'key': (size) 'data'\n");

       key = gdbm_firstkey ( dbfile );
       while (key.dptr) {
           data =  gdbm_fetch ( dbfile, key );
           recordCount++;
           npKey=0;

           if(strlen(key.dptr) > 15) wideMode=1;

           for(i=0; i<strlen(key.dptr); i++) {
               if((key.dptr[i] < ' ') || (key.dptr[i] > '~')) {
                   key.dptr[i] = '.';
                   npKey=1;
               }
           }

           if(wideMode == 1) {
               if(snprintf(buf, sizeof(buf), "'%s'", key.dptr) < 0) {
                   buf[39]='.';
                   buf[40]='.';
                   buf[41]='.';
                   buf[42]='\0';
               }
           } else {
               snprintf(buf, sizeof(buf), "'%s'", key.dptr);
           }

           printf ("%s ", npKey==0 ? " " : "*");
           if(wideMode == 1) {
               printf ("%-43s", buf);
           } else {
               printf ("%-18s", buf);
           }
           printf (": (%3d) ", data.dsize);

           foundBinary=0;
           for(i=0;i<data.dsize; i++) {
             if((data.dptr[i] < ' ') || (data.dptr[i] > '~')) {
               foundBinary=1;
               break;
             }
           }
               
           if(foundBinary == 1) {
             for(i=0; i<data.dsize; i += 16) {
               if(i>0)
                 if(wideMode == 1) {
                     printf("                                                     ");
                 } else {
                     printf("                            ");
                 }

               for(j=0; j<16; j++) {
                 if(i+j >= data.dsize) {
                   printf("  ");
                 } else {
                   outC = 0xff & data.dptr[i+j];
                   printf("%02x", outC);
                 }
                 if(j%4 == 3) 
                   printf(" ");
               }

               printf("  ");
               for(j=0; j<16; j++) {
                 if((data.dptr[i+j] < ' ') || (data.dptr[i+j] > '~')) {
                   printf(".");
                 } else {
                   printf("%c", data.dptr[i+j]);
                 }
               }

               printf("\n");
             }
           } else {
             printf ("'%s'\n", key.dptr, data.dptr);
           }
           free (data.dptr);
           key = gdbm_nextkey ( dbfile, key );
       }
       printf ("Records read: %d\n", recordCount);
    } else {

       printf ("Searching file %s for key '%s'...\n", argv[1], argv[2]);
       printf ("'key': 'data'\n");
       key.dsize = strlen (argv[2]) + 1;
       key.dptr = argv[2];
       data = gdbm_fetch (dbfile, key);
       if (data.dptr) {
           printf ("'%15s': '%s'\n", key.dptr, data.dptr);
           free (data.dptr);
       } else {
           printf ("Key %s not found.\n", argv[2]);
       }
   }
   gdbm_close (dbfile);

   printf("Note: Records preceded by a '*' have keys with non-printable characters "
          "printed as '.'\n");
}

